###########################################################################
# $Id: Storage.p,v 1.14 2007-12-04 11:45:51 misha Exp $
# base class for set/get data
# like hashfile but allow to set/get/delete not only strings and you can specify expires during object creation

# limitations:
#	- not accept tables as values of hash
#	- image and user classes not accepted
#	- works only till year 2037


@CLASS
Storage


@USE
Lib.p
Convert.p


###########################################################################
@create[sStorageName;hParam]
^if(!def $sStorageName){
	^self._throw[^$sStorageName must be specified.]
}

$hParam[^hash::create[$hParam]]
$self.uExpires[$hParam.uExpires]
$self.sStorageName[$sStorageName]

^self._init[$hParam]
#end @create[]



###########################################################################
@get[sKey][hValue]
$result[]
^if(def $sKey){
	^self._open[]
	$hValue[^self._get[$sKey]]
	^switch[$hValue.sType]{
		^case[]{$result[]}
		^case[double;int]{$result(^self._convert2object[$hValue.sType;$hValue.uValue])}
		^case[DEFAULT]{$result[^self._convert2object[$hValue.sType;$hValue.uValue]]}
	}
	^self._release[]
}{
	$result[]
}
#end @get[]



###########################################################################
@set[sKey;hParam][hValue]
^if(def $sKey){
	$hValue[^self._prepare[$hParam]]
	^if(def $hValue.sType){
		^self._open[]
		^self._set[$sKey;$hValue]
		^self._maintenance[]
		^self._release[]
	}{
		^self.delete[$sKey]
	}
}
$result[]
#end @set[]



###########################################################################
@delete[sKey]
^if(def $sKey){
	^self._open[]
	^self._delete[$sKey]

	^self._maintenance[]
	^self._release[]
}
$result[]
#end @delete[]



###########################################################################
@cleanup[]
^self._open[]
^self._cleanup[]
^self._release[]
$result[]
#end @cleanup[]



###########################################################################
# private

@_throw[sSource;sComment]
^throw[$sClassName;$sSource;$sComment]


@_init[hParam]
$result[]


@_open[]
$result[]


@_get[sKey]
^self._throw[Method @_get must be implemented.]


@_set[sKey;hParam]
^self._throw[Method @_set must be implemented.]


@_release[]
$result[]


@_cleanup[]
$result[]


@_maintenance[]
^if(^math:random(100) < 3){
	^self._cleanup[]
}
$result[]


@_normalize[sKey]
$result[^math:md5[$sKey]]


@_prepare[hParam][sType]
$hParam[^hash::create[$hParam]]
$sType[^if($hParam.value is "double" || $hParam.value is "int"){^Lib:getType($hParam.value)}{^Lib:getType[$hParam.value]}]
# we must ignore empty strings but store empty tables/hashes
^if(!def $hParam.value && $sType eq "string"){$sType[]}
$result[
	$.sType[$sType]
	$.uValue[^if(def $sType){^self._convert2saveable[$sType;$hParam.value]}]
	$.uExpires[^if(def $hParam.expires){$hParam.expires}{$self.uExpires}]
]
